///\file

/******************************************************************************
The MIT License(MIT)

Embedded Template Library.
https://github.com/ETLCPP/etl
https://www.etlcpp.com

Copyright(c) 2020 John Wellbelove

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files(the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions :

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/

#ifndef ETL_IREFERENCE_COUNTED_MESSAGE_POOL_INCLUDED
#define ETL_IREFERENCE_COUNTED_MESSAGE_POOL_INCLUDED

#include "platform.h"

namespace etl
{
  class ireference_counted_message;

  //***************************************************************************
  /// Interface for a reference counted message pool.
  //***************************************************************************
  class ireference_counted_message_pool
  {
  public:

    virtual ~ireference_counted_message_pool() {}
    virtual void release(const etl::ireference_counted_message& msg) = 0;

  protected:

    //***************************************************************************
    /// The pool lock function.
    /// Override to add thread or interrupt locking to the pool.
    //***************************************************************************
    virtual void lock()
    {
      // The default implementation does nothing.
    }

    //***************************************************************************
    /// The pool unlock function.
    /// Override to add thread or interrupt unlocking to the pool.
    //***************************************************************************
    virtual void unlock()
    {
      // The default implementation does nothing.
    }
  };
}

#endif

