///\file

/******************************************************************************
The MIT License(MIT)

Embedded Template Library.
https://github.com/ETLCPP/etl
https://www.etlcpp.com

Copyright(c) 2014 John Wellbelove

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files(the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions :

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/

#ifndef ETL_PARAMETER_INCLUDED
#define ETL_PARAMETER_INCLUDED

#include "platform.h"
#include "type_traits.h"

namespace etl
{
  //*************************************************************************
  /// Determine how to pass parameters.
  //*************************************************************************
  template <typename T>
  struct parameter_type
  {
    /// By default fundamental and pointer types are passed by value.
    typedef typename etl::conditional<etl::is_fundamental<T>::value || etl::is_pointer<T>::value,
                                         T,
                                         const T&>::type type;
  };

#if ETL_USING_CPP11
  template <typename T>
  using parameter_type_t = typename etl::parameter_type<T>::type;
#endif
}

#endif
